#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.31
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import { select, Separator, confirm, input } from '@inquirer/prompts';
import chalk from 'chalk';
import dayjs from 'dayjs';
import to from 'await-to-done';
import { createArgs, printQrcode, readQrcode } from '@gitmars/utils';
import { getPreAuthQrCode, getAuthorizerListWithAllDetail, getAuditStatus, undoAudit, getTrialQrCode, bindTester, unbindTester } from '@gitmars/api';
import lang from './common/local.mjs';
import { cmdConfig } from './conf/miniprogram.mjs';

const { t } = lang;
const { args, options } = cmdConfig;
program.name("gitm miniprogram").usage("[miniprogram] [-k --keyword [keyword]]").description(t("miniprogram command"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (miniprogram, opt) => {
  if (miniprogram === "auth") {
    const authUrl = await getPreAuthQrCode();
    await printQrcode(authUrl);
  } else if (!miniprogram) {
    const [, list = []] = await to(
      getAuthorizerListWithAllDetail({ limit: 500 }).then(({ list: list2 }) => {
        if (!opt.keyword)
          return list2;
        return list2.filter((item) => item.authorizer_info.nick_name.includes(opt.keyword));
      })
    );
    [, miniprogram = ""] = await to(
      select({
        message: t("Select the application to build"),
        choices: list.map(({ authorizer_info: info, authorizer_appid }) => ({
          name: info.nick_name,
          value: authorizer_appid
        }))
      })
    );
  }
  if (!miniprogram)
    process.exit(0);
  const [, action = ""] = await to(
    select({
      message: t("Please select the operation you want?"),
      choices: [
        new Separator(" === 1. " + t("Audit") + " === "),
        {
          name: t("Audit Status"),
          value: "audit_status"
        },
        {
          name: t("Undo Audit"),
          value: "undo_audit"
        },
        {
          name: t("Submit Audit"),
          value: "submit_audit",
          disabled: true
        },
        new Separator(" === 2. " + t("Common") + " === "),
        {
          name: t("Trial Qrcode"),
          value: "trial_qrcode"
        },
        {
          name: t("Bind Tester"),
          value: "bind_tester"
        },
        {
          name: t("Unbind Tester"),
          value: "unbind_tester"
        },
        new Separator(" === " + t("Exit") + " === "),
        { name: "exit", value: "exit" },
        new Separator()
      ]
    })
  );
  if (action === "audit_status") {
    const [, data] = await to(getAuditStatus(miniprogram));
    if (data) {
      const colorList = ["bgGreen", "bgRed", "bgBlack", "bgBlack", "bgYellow"];
      const tag = ["\u5BA1\u6838\u6210\u529F", "\u5BA1\u6838\u88AB\u62D2\u7EDD", "\u5BA1\u6838\u4E2D", "\u5DF2\u64A4\u56DE", "\u5BA1\u6838\u5EF6\u540E"][data.status];
      let txt = chalk[colorList[data.status]](tag);
      if ([1, 4].includes(data.status) && data.reason)
        txt += ": " + data.reason;
      if (data.time)
        txt += "\n" + dayjs(data.time).format("YYYY-MM-DD HH:mm");
      sh.echo(txt);
    }
  } else if (action === "undo_audit") {
    const [, answer] = await to(
      confirm({
        message: t("Confirm undo audit"),
        default: false
      })
    );
    if (answer) {
      await undoAudit(miniprogram);
      sh.echo(chalk.green(t("Implementation success")));
    }
  } else if (action === "submit_audit") ; else if (action === "trial_qrcode") {
    const [, path = ""] = await to(
      input({
        message: t("Enter path of miniprogram"),
        transformer: (val) => val.trim()
      }).then((val) => val.trim())
    );
    const [, codeImage = ""] = await to(getTrialQrCode({ path, authorizer_appid: miniprogram }));
    printQrcode(await readQrcode(codeImage));
  } else if (action === "bind_tester") {
    const [, wechatid = ""] = await to(
      input({
        message: t("Enter wechat account"),
        transformer: (val) => val.trim()
      }).then((val) => val.trim())
    );
    if (wechatid) {
      await bindTester({ wechatid, authorizer_appid: miniprogram });
      sh.echo(chalk.green(t("Implementation success")));
    }
  } else if (action === "unbind_tester") {
    const [, userstr = ""] = await to(
      input({
        message: t("Enter wechat userstr"),
        transformer: (val) => val.trim()
      }).then((val) => val.trim())
    );
    if (userstr) {
      await unbindTester({ userstr, authorizer_appid: miniprogram });
      sh.echo(chalk.green(t("Implementation success")));
    }
  }
});
program.parse(process.argv);
